package ch.hepia.exos;

/**
 * A small app to test the {@link ch.hepia.exos.MySet} implemented.
 */
public class App {
    /**
     * The norm of a given set.
     * 
     * @param set The set to compute the norm on.
     * @return The norm of the {@link ch.hepia.exos.MySet} = sqrt(e1^2 + e2^2 + ...
     *         + en^2)
     */
    public static Double norm(MySet<Double> set) {
        return Math.sqrt(set.mapReduce(0.0, (acc, value) -> acc + Math.pow(value, 2)));
    }

    /**
     * Computes the factorial of n.
     * 
     * @param n The argument of the factorial function
     * @return %he factorial of n = n! = 1*2*...*n.
     */
    public static Integer factorial(int n) {
        return MySet.range(1, n + 1).reduce(1, (x, y) -> x * y);
    }

    /**
     * Computes all the primes number lower than a
     * given number.
     * 
     * @param maxExclusive The upper bound.
     * @return A {@link ch.hepia.exos.MySet} containing prime numbers.
     */
    public static MySet<Integer> primesUnder(int maxExclusive) {
        if (maxExclusive < 2) {
            return new MySet<>();
        } else if (maxExclusive == 2) {
            return MySet.of(2);
        } else {
            return MySet
                    .range(2, maxExclusive)
                    .filter(i -> {
                        return MySet
                                .range(2, ((int) Math.sqrt(i)) + 1)
                                .all(j -> i % j != 0);
                    });
        }
    }

    /**
     * The main function to try the {@link ch.hepia.exos.MySet} collection
     * implemented.
     * 
     * @param args the args passed to the program.
     */
    public static void main(String[] args) {
        MySet<Double> set = MySet.of(3.0, 4.0, 5.0);
        System.out.println("Norm of " + set.toString() + "=" + norm(set));
        System.out.println("Primes under 30 = " + primesUnder(30).toString());
    }
}
