# Questions stream

## Introduction

Pour cet exercice, vous devrez répondre à une série de questions sur une base de donnée fiscale factice en vous servant des streams. 

Vous devez commencer par télécharger puis décompresser les archives suivantes :

small : https://hessoit-my.sharepoint.com/:u:/g/personal/michael_elkharro_hes-so_ch/Ebpm5kAxb19AgMIn0fWotYwB9vlAh9IgV_28M0d7BgG_yg?e=rOpeCh
full  : https://hessoit-my.sharepoint.com/:u:/g/personal/michael_elkharro_hes-so_ch/Eect8iTAimhKgwBLCo92uUABbvrfP7wiW8uSNI7IwSCKhg?e=jue8nH


Le package fourni avec cet énoncé contient une classe FiscalDB, ainsi qu'une classe Utils. La classe FiscalDB expose un constructueur qui prend en paramètre le chemin sur votre ordinateur de l'un des deux fichiers CSV téléchargés et décompressés. La classe propose également 3 méthodes :

```java
public Stream<Person> stream();

public Stream<Person> parallelStream();

public int size();
```

Vous êtes libre d'utiliser `stream` ou `parallelStream` à votre convenance.

La classe imbriquée FiscalDB.Person dispose des méthodes suivantes :

```java
public long getId();

public String getFirstName();

public String getLastName();

public Optional<Person> getSpouse();

public boolean hasSpouse();

public Person getSpouseOrElseThrow();

public double getIncomes();

public double getAssets();

public String getCity();
```

Vous devrez utiliser uniquement ces méthodes pour répondre aux questions.

## Aide

Il est possible que vous rencontriez des problèmes avec le fichier CSV complet. Dans ce cas, vous pouvez utiliser le fichier réduit.

Vu la taille du fichier, vous pouvez rencontrez des erreurs du type :

```
thread Thread[ForkJoinPool.commonPool-worker-4,5,ch.hepia.exos.App] was interrupted but is still alive after waiting at least 15000msecs
...
java.lang.IllegalThreadStateException
```

La solution la plus simple est de lancer le programme avec la commande suivante

```bash
mvn clean compile exec:java -Dexec.cleanupDaemonThreads=false
```



## Question faciles :

1. Quel est le revenu moyen et médian de la population?
   1. Rappel la médiane d'un ensemble $K$ est la valeur $\tilde x$ ayant autant d’éléments plus petits que d'éléments plus grands $\in K$. Par exemple : 3.5 est la médiane de $[7,2,5,6,4,1,0,3]$ et 2 est la médiane de $[0,1,2,3,4]$.
2. Quelle est la personne qui a les plus gros revenus?
3. Qui est la personne avec la dette la plus importante?
4. Combien de personnes sont endettées?
5. Combien de personnes gagnent plus d'un million?
6. Combien de personnes sont mariées?
7. Est-ce que les 80% qui gagnent le moins, gagnent plus ou moins que les 5% qui gagnent le plus?
8. Combien de personnes sont multimillionnaires?
   1.  Précision : patrimoine >= 2'000'000
9. Quelle est la personne qui a les plus petits revenus, parmi les multimillionnaires?
10. Est-ce que le revenu médian des personnes mariées est plus grand ou plus petit que celui des personnes célibataires?

## Questions moins faciles :

À partir de maintenant on défini la richesse d'une personne comme étant : $\textit{Richesse} = \textit{Salaire}+0.1\cdot\textit{Patrimoine}$.

1. Quelle est la médiane des richesses?
2. Quelle est la ville ayant le plus d'habitant?
3. Si on regroupe la population par la première lettre de leur prénom, quel groupe a la plus grande richesse?
   1. Précision : Réponse possible, les gens dont le prénom commence par 'Å'.
4. Quelle est la ville contenant le plus de célibataires?
5. Quelle ville possède le plus de riches ?
   1. Précision : une personne riche est une personne qui a une richesse > à 95% de la population.

## Questions pas faciles :

En prenant en compte les tableaux d'imposition suivants :

1. Impôt sur le revenu

   | Revenus (en médiane de revenus de la ville) | Taux |
   | ------------------------------------------- | ---- |
   | < 0.5 fois la médiane                       | 0%   |
   | <= 1 fois la médiane                        | 12%  |
   | > 1 fois la médiane                         | 33%  |
   | > 5 fois la médiane                         | 50%  |

2. Impôt sur la fortune

   | Fortune (en médiane de fortunes de la ville) | Taux |
   | -------------------------------------------- | ---- |
   | < 0                                          | -10% |
   | <= 1 fois la médiane                         | 1%   |
   | > 1 fois la médiane                          | 3%   |
   | > 5 fois la médiane                          | 7%   |

Pour calculer l'impôt d'une personne, on se sert de la formule suivante :
$$
\textit{Impôt}_{\textit{Revenu}} =& \textit{Taux}_{\textit{Revenu}}* \textit{Revenu}\\
\textit{Impôt}_{\textit{Fortune}} =& \max(-0.25\cdot\textit{Impôt}_{\textit{Revenu}},\textit{Taux}_{\textit{Fortune}}* \textit{Fortune})\\
\textit{Impôt}_{\textit{Total}} =& \textit{Impôt}_{\textit{Revenu}} + \textit{Impôt}_{\textit{Fortune}}
$$
Si on calcule l'impôt pour un couple, on fait la somme de leurs revenus et respectifs, puis la somme de leurs fortunes et on applique la même formule. Ce qui donne :
$$
\textit{Revenu} =& Revenu_{époux 1} + Revenu_{époux 2}\\
\textit{Fortune} =& Fortune_{époux 1} + Fortune_{époux 2}\\
\textit{Impôt}_{\textit{Revenu}} =& \textit{Taux}_{\textit{Revenu}}* \textit{Revenu}\\
\textit{Impôt}_{\textit{Fortune}} =& \max(-0.25\cdot\textit{Impôt}_{\textit{Revenu}},\textit{Taux}_{\textit{Fortune}}* \textit{Fortune})\\
\textit{Impôt}_{\textit{Total}}\ (\textit{par époux}) =& 0.5*(\textit{Impôt}_{\textit{Revenu}} + \textit{Impôt}_{\textit{Fortune}})
$$

1. Donnez la liste des villes par ordre croissant de la médiane des richesses.
2. Séparez les personnes en 5 classes de richesses et comptez le nombre de personnes dans chacune des classes.
   1. Précision :  les classes de richesses sont les suivantes :
      1.  richesses < 1 fois la médiane
      2. 1 fois la médiane <= richesses < 3 fois la médiane
      3. 3 fois la médiane <= richesses < 6 fois la médiane
      4. 6 fois la médiane <= richesses < 12 fois la médiane
      5. 12 fois la médiane <= richesses
3. Quelle est la ville qui paye le plus d'impôt?
4. Donnez la liste des villes par ordre croissant de justesse moyenne (au sens de la formule donnée ci-dessous).
   1. Précision : On calcule le taux de justesse pour une personne comme étant $\frac{\textit{Impôt}_{\textit{Total}}}{Richesse}$

## Question ultime :

1. Quel pourcentage des mariages contient des époux :
   1. séparés de plusieurs classes de richesses (voir la question pas facile n°2)?
   2. séparés d'une seule classe de richesses?
   3. ayant la même classe de richesses?