package ch.hepia.exos;

import java.util.stream.DoubleStream;
import ch.hepia.exos.FiscalDB.Person;

public class Utils {
    public static double computeMedian(int size, DoubleStream s) {
        int mid = size / 2;
        if (size % 2 == 0) {
            return s.sorted()
                    .skip(mid - 1)
                    .limit(2)
                    .average()
                    .getAsDouble();
        } else {
            return s
                    .sorted()
                    .skip(mid)
                    .findFirst()
                    .getAsDouble();
        }
    }

    public static double computeWealth(Person p) {
        return p.getIncomes() + 0.1 * p.getAssets();
    }

    public static double computeTaxes(Person p, double medianIncomes, double medianAssets) {
        double incomes = p.getIncomes();
        double assets = p.getAssets();
        if (p.hasSpouse()) {
            incomes += p.getSpouseOrElseThrow().getIncomes();
            assets += p.getSpouseOrElseThrow().getAssets();
        }
        double incomesRate;
        if (incomes < 0.5 * medianIncomes) {
            incomesRate = 0;
        } else if (incomes <= medianIncomes) {
            incomesRate = 0.12;
        } else if (incomes <= 5 * medianIncomes) {
            incomesRate = 0.33;
        } else { // incomes > 5*median
            incomesRate = 0.5;
        }

        double assetsRate;
        if (assets < 0) {
            assetsRate = -0.1;
        } else if (assets <= medianAssets) {
            assetsRate = 0.01;
        } else if (assets <= 5 * medianIncomes) {
            assetsRate = 0.03;
        } else { // assets > 5*median
            assetsRate = 0.07;
        }

        double incomesTaxes = incomesRate * incomes;
        double assetsTaxes = Math.max(-0.25 * incomesTaxes, assetsRate * assets);
        double taxes = incomesTaxes + assetsTaxes;
        if (p.hasSpouse()) {
            taxes *= 0.5;
        }
        return taxes;
    }
}
