public class Carte {
    private final int rang;
    private final Couleur couleur;
    private  int force;

    public Carte(Couleur couleur, int rang){
        if (rang < 0) {
            throw new IllegalArgumentException("This field cannot be negative");
        } else if (couleur != Couleur.coeur &&couleur != Couleur.pique &&couleur != Couleur.carreau &&couleur != Couleur.trefle &couleur != Couleur.none) {
            throw new IllegalArgumentException("Please choose one of the 4 color options");
        } else if (rang==0 || rang==14) { //If this is a joker or a cavalier, the color by default will be none
            this.rang=rang;
            this.couleur=Couleur.none;
        } else{
            this.couleur=couleur;
            this.rang = rang;
        }
    }
    public Carte(Couleur couleur, int rang, int force){
        if (rang < 0) {
            throw new IllegalArgumentException("This field cannot be negative");
        } else if (couleur != Couleur.coeur &&couleur != Couleur.pique &&couleur != Couleur.carreau &&couleur != Couleur.trefle &couleur != Couleur.none) {
            throw new IllegalArgumentException("Please choose one of the 4 color options");
        } else if (rang==0 || rang==14) { //If this is a joker or a cavalier, the color by default will be none
            this.rang=rang;
            this.couleur=Couleur.none;
            this.force= force;
        } else{
            this.couleur=couleur;
            this.rang = rang;
            this.force= force;
        }
    }

    public Couleur getCouleur(){
        return couleur;
    }

    public int getRang() {
        return rang;
    }

    public int getForce() {
        return force;
    }

    public String getNomCouleur(){
        return getCouleur().name();
    }

    public String getNomRang(){
        int rang = getRang();
        String nomRang;
        switch(rang) {
            case 0:
                nomRang="Joker";
                break;
            case 1:
                nomRang="As";
                break;
            case 11:
                nomRang="Valet";
                break;
            case 12:
                nomRang="Dame";
                break;
            case 13:
                nomRang="Roi";
                break;
            case 14:
                nomRang="Cavalier";
                break;
            default:
                nomRang = String.valueOf(rang);
        }
        return nomRang;
    }

    public String getNomComplet(){
        return couleur == Couleur.none? getNomRang(): getNomRang() + " de " + getNomCouleur();
    }

    public String toString() {
        return getNomComplet();
    }
}
