import java.util.Random;
public class JeuDeCarte {
    private Carte[] tab;

    public JeuDeCarte() {

    }

    /**
     * @param type the enum type to convert into an int
     * @return the number of cards per color-> depends on the kind of game
     */
    public static int convertTypeToInt(Type type){
        return (type == Type.standard) ? 8 : 13;
    }

    /**
     * @param val the int value to convert into a type
     * @return the type conresponding to a value. Return null if the val is not one of the possibilities
     */
    public static Type convertIntToType(int val){
        switch (val){
            case (1):
                return Type.standard;
            case (2):
                return Type.standard13;
            case (3):
                return Type.standard13Plus2jokers;
            case (4):
                return Type.standard13Plus4cavaliers;
            case (5):
                return Type.tarot;
            default:
                System.out.println("\nThis kind of game is not available");
        }
        return null;
    }

    /**
     * @param type the type of game cards to create
     * @return a game cards
     */
    public static JeuDeCarte createPaquet(Type type) {
        JeuDeCarte paquet = new JeuDeCarte();
        int valType = convertTypeToInt(type);
        switch (type){
            case standard13Plus2jokers:
                paquet.tab = new Carte[(Couleur.values().length -1) * valType + 2]; //couleur values -1 to avoid the none color * n cards per color + 2 jokers
                break;
            case standard13Plus4cavaliers:
                paquet.tab = new Carte[(Couleur.values().length -1) * valType + 4]; //couleur values -1 to avoid the none color * n cards per color + 4 cavaliers
                break;
            case tarot:
                //TODO
                System.out.println("\nThe tarot game is not available");
                break;
            default:
                paquet.tab = new Carte[(Couleur.values().length -1) * valType]; //couleur values -1 to avoid the none color * n cards per color
                break;
        }
        int pos=0, i;
        for (Couleur color : Couleur.values()) {
            if(color!=Couleur.none){
                for (i = 1; i <= valType; i++) {
                    Carte carte = new Carte(color, i);
                    paquet.tab[pos] = carte;
                    pos++;
                }
            }
        }
        if (type==Type.standard13Plus2jokers){
            paquet.tab[pos] = new Carte(Couleur.none, 0);
            paquet.tab[pos+1] = new Carte(Couleur.none, 0);
        } else if (type==Type.standard13Plus4cavaliers) {
            for (int j=0;j<4;j++){
                paquet.tab[pos+j] = new Carte(Couleur.none, 14);
            }
        }
        System.out.println("Deck created successfully");
        return paquet;
    }

    public void shuffle() {
        int n = tab.length;
        Random random = new Random();
        for (int i = n - 1; i > 0; i--) {
            int j = random.nextInt(i + 1);
            Carte temp = tab[i];
            tab[i] = tab[j];
            tab[j] = temp;
        }
    }

    /**
     * @param handSize the number of cards per hands
     * @param jeu the card game used to create the hands
     * @return a tab of shuffled hands
     */
    public static JeuDeCarte[] createHand(int handSize, JeuDeCarte jeu){
        jeu.shuffle();
        int cptImpaire = 1; int cptPaire = 0;
        JeuDeCarte hand1 = new JeuDeCarte();JeuDeCarte hand2 = new JeuDeCarte();
        hand1.tab = new Carte[handSize]; hand2.tab = new Carte[handSize];
        JeuDeCarte[] tabHand = {hand1,hand2};
        for (int i = 0; i < handSize; i++) {
            hand1.tab[i] = jeu.tab[cptPaire];
            hand2.tab[i] = jeu.tab[cptImpaire];
            cptPaire +=2;
            cptImpaire +=2;
            }
        return tabHand;
    }

    /**
     * @param n the number of decks to create
     * @return a tab of shuffled decks
     */
    public static JeuDeCarte[] paquet(int n){
        JeuDeCarte[] paquet = new JeuDeCarte[n];
        Type input;
        for (int i = 0; i < n; i++) {
            System.out.println("\nDeck n°"+(i+1));
            ui.displayGame();
            input = JeuDeCarte.convertIntToType(ui.inputUser());
            paquet[i] = createPaquet(input);
            paquet[i].shuffle();
        }
        return paquet;
    }

    public String toString() {
        String result="";
        for (Carte carte:tab) {
            result = carte.toString();
            System.out.println(carte);
        }
        return result;
    }
}